@@ -8,6 +8,8 @@
 #include "stdinc.h"
 #include "modules.h"
 #include "client.h"
+#include "privilege.h"
+#include "s_newconf.h"
 #include "ircd.h"
 
 static int _modinit(void);
@@ -36,9 +38,10 @@ static int eb_oper(const char *data, struct Client *client_p,
 
 	(void)chptr;
 	(void)mode_type;
-	/* perhaps use data somehow? (opernick/flags?) */
-	/* so deny any bans with data for now */
+
 	if (data != NULL)
-		return EXTBAN_INVALID;
+		/* $o:admin or whatever */
+		return HasPrivilege(client_p, data) ? EXTBAN_MATCH : EXTBAN_NOMATCH;
+
 	return IsOper(client_p) ? EXTBAN_MATCH : EXTBAN_NOMATCH;
 }
