@@ -761,11 +761,32 @@ mangle_mapped_sockaddr(struct sockaddr *in)
  * rb_listen() - listen on a port
  */
 int
-rb_listen(rb_fde_t *F, int backlog)
+rb_listen(rb_fde_t *F, int backlog, int defer_accept)
 {
+	int result;
+
 	F->type = RB_FD_SOCKET | RB_FD_LISTEN;
-	/* Currently just a simple wrapper for the sake of being complete */
-	return listen(F->fd, backlog);
+	result = listen(F->fd, backlog);
+
+#ifdef TCP_DEFER_ACCEPT
+	if (defer_accept && !result)
+	{
+		setsockopt(F->fd, IPPROTO_TCP, TCP_DEFER_ACCEPT, &backlog, sizeof(int));
+	}
+#endif
+#ifdef SO_ACCEPTFILTER
+	if (defer_accept && !result)
+	{
+		struct accept_filter_arg afa;
+
+		memset(&afa, '\0', sizeof afa);
+		rb_strlcpy(afa.af_name, "dataready", sizeof afa.af_name);
+		(void)setsockopt(F->fd, SOL_SOCKET, SO_ACCEPTFILTER, &afa,
+				sizeof afa);
+	}
+#endif
+
+	return result;
 }
 
 void
@@ -1310,7 +1331,7 @@ inet_ntop6(const unsigned char *src, char *dst, unsigned int size)
 	{
 		return (NULL);
 	}
-	return strcpy(dst, tmp);
+	return memcpy(dst, tmp, tp - tmp);
 }
 #endif
 
