@@ -92,8 +92,6 @@ static int build_target_list(int p_or_n, const char *command,
 			     struct Client *source_p, const char *nicks_channels, const char *text);
 
 static int flood_attack_client(int p_or_n, struct Client *source_p, struct Client *target_p);
-static int flood_attack_channel(int p_or_n, struct Client *source_p,
-				struct Channel *chptr, char *chname);
 
 /* Fifteen seconds should be plenty for a client to reply a ctcp */
 #define LARGE_CTCP_TIME 15
@@ -522,7 +520,7 @@ msg_channel(int p_or_n, const char *command,
 		   !flood_attack_channel(p_or_n, source_p, chptr, chptr->chname))
 		{
 			if (p_or_n != NOTICE && *text == '\001' &&
-					strncasecmp(text + 1, "ACTION", 6))
+					strncasecmp(text + 1, "ACTION ", 7))
 			{
 				if (chptr->mode.mode & MODE_NOCTCP)
 				{
@@ -632,6 +630,7 @@ static void
 msg_channel_flags(int p_or_n, const char *command, struct Client *client_p,
 		  struct Client *source_p, struct Channel *chptr, int flags, const char *text)
 {
+	char text2[BUFSIZE];
 	int type;
 	char c;
 
@@ -653,6 +652,40 @@ msg_channel_flags(int p_or_n, const char *command, struct Client *client_p,
 			source_p->localClient->last = rb_current_time();
 	}
 
+	if(chptr->mode.mode & MODE_NOCOLOR)
+	{
+		rb_strlcpy(text2, text, BUFSIZE);
+		strip_colour(text2);
+		text = text2;
+		if (EmptyString(text))
+		{
+			/* could be empty after colour stripping and
+			 * that would cause problems later */
+			if(p_or_n != NOTICE)
+				sendto_one(source_p, form_str(ERR_NOTEXTTOSEND), me.name, source_p->name);
+			return;
+		}
+	}
+
+	if (p_or_n != NOTICE && *text == '\001' &&
+			strncasecmp(text + 1, "ACTION ", 7))
+	{
+		if (chptr->mode.mode & MODE_NOCTCP)
+		{
+			sendto_one_numeric(source_p, ERR_CANNOTSENDTOCHAN,
+					   form_str(ERR_CANNOTSENDTOCHAN), chptr->chname);
+			return;
+		}
+		else if (rb_dlink_list_length(&chptr->locmembers) > (unsigned)(GlobalSetOptions.floodcount / 2))
+		{
+			/* This overestimates the number of users the CTCP
+			 * is being sent to, so large_ctcp_sent might be
+			 * set inappropriately. This should not be a problem.
+			 */
+			source_p->large_ctcp_sent = rb_current_time();
+		}
+	}
+
 	sendto_channel_flags(client_p, type, source_p, chptr, "%s %c%s :%s",
 			     command, c, chptr->chname, text);
 }
@@ -887,63 +920,6 @@ flood_attack_client(int p_or_n, struct Client *source_p, struct Client *target_p
 	return 0;
 }
 
-/*
- * flood_attack_channel
- * inputs       - flag 0 if PRIVMSG 1 if NOTICE. RFC
- *                says NOTICE must not auto reply
- *              - pointer to source Client 
- *		- pointer to target channel
- * output	- 1 if target is under flood attack
- * side effects	- check for flood attack on target chptr
- */
-static int
-flood_attack_channel(int p_or_n, struct Client *source_p, struct Channel *chptr, char *chname)
-{
-	int delta;
-
-	if(GlobalSetOptions.floodcount && MyClient(source_p))
-	{
-		if((chptr->first_received_message_time + 1) < rb_current_time())
-		{
-			delta = rb_current_time() - chptr->first_received_message_time;
-			chptr->received_number_of_privmsgs -= delta;
-			chptr->first_received_message_time = rb_current_time();
-			if(chptr->received_number_of_privmsgs <= 0)
-			{
-				chptr->received_number_of_privmsgs = 0;
-				chptr->flood_noticed = 0;
-			}
-		}
-
-		if((chptr->received_number_of_privmsgs >= GlobalSetOptions.floodcount)
-		   || chptr->flood_noticed)
-		{
-			if(chptr->flood_noticed == 0)
-			{
-				sendto_realops_snomask(SNO_BOTS, *chptr->chname == '&' ? L_ALL : L_NETWIDE,
-						     "Possible Flooder %s[%s@%s] on %s target: %s",
-						     source_p->name, source_p->username,
-						     source_p->orighost,
-						     source_p->servptr->name, chptr->chname);
-				chptr->flood_noticed = 1;
-
-				/* Add a bit of penalty */
-				chptr->received_number_of_privmsgs += 2;
-			}
-			if(MyClient(source_p) && (p_or_n != NOTICE))
-				sendto_one(source_p,
-					   ":%s NOTICE %s :*** Message to %s throttled due to flooding",
-					   me.name, source_p->name, chptr->chname);
-			return 1;
-		}
-		else
-			chptr->received_number_of_privmsgs++;
-	}
-
-	return 0;
-}
-
-
 /*
  * handle_special
  *
