@@ -138,7 +138,7 @@ mr_nick(struct Client *client_p, struct Client *source_p, int parc, const char *
 		*s = '\0';
 
 	/* copy the nick and terminate it */
-	rb_strlcpy(nick, parv[1], sizeof(nick));
+	rb_strlcpy(nick, parv[1], ConfigFileEntry.nicklen);
 
 	/* check the nickname is ok */
 	if(!clean_nick(nick, 1))
@@ -201,7 +201,7 @@ m_nick(struct Client *client_p, struct Client *source_p, int parc, const char *p
 		flood_endgrace(source_p);
 
 	/* terminate nick to NICKLEN, we dont want clean_nick() to error! */
-	rb_strlcpy(nick, parv[1], sizeof(nick));
+	rb_strlcpy(nick, parv[1], ConfigFileEntry.nicklen);
 
 	/* check the nickname is ok */
 	if(!clean_nick(nick, 1))
@@ -566,7 +566,7 @@ clean_nick(const char *nick, int loc_client)
 	}
 
 	/* nicklen is +1 */
-	if(len >= NICKLEN)
+	if(len >= NICKLEN && len >= ConfigFileEntry.nicklen)
 		return 0;
 
 	return 1;
@@ -728,7 +728,7 @@ change_local_nick(struct Client *client_p, struct Client *source_p,
 			     source_p->name, nick, source_p->username, source_p->host);
 
 	/* send the nick change to the users channels */
-	sendto_common_channels_local(source_p, ":%s!%s@%s NICK :%s",
+	sendto_common_channels_local(source_p, NOCAPS, ":%s!%s@%s NICK :%s",
 				     source_p->name, source_p->username, source_p->host, nick);
 
 	/* send the nick change to servers.. */
@@ -789,7 +789,7 @@ change_remote_nick(struct Client *client_p, struct Client *source_p,
 		monitor_signoff(source_p);
 	}
 
-	sendto_common_channels_local(source_p, ":%s!%s@%s NICK :%s",
+	sendto_common_channels_local(source_p, NOCAPS, ":%s!%s@%s NICK :%s",
 				     source_p->name, source_p->username, source_p->host, nick);
 
 	if(source_p->user)
