@@ -75,7 +75,7 @@ mr_server(struct Client *client_p, struct Client *source_p, int parc, const char
 	const char *name;
 	struct Client *target_p;
 	int hop;
-	struct Capability *cap;
+	unsigned int required_mask;
 
 	name = parv[1];
 	hop = atoi(parv[2]);
@@ -110,20 +110,13 @@ mr_server(struct Client *client_p, struct Client *source_p, int parc, const char
 	}
 
 	/* check to ensure any "required" caps are set. --nenolod */
-	for (cap = captab; cap->name; cap++)
+	/* XXX: show required CAPABs. */
+	required_mask = capability_index_get_required(serv_capindex);
+	if (required_mask && !(client_p->localClient->caps & required_mask))
 	{
-		if (!cap->required)
-			continue;
+		exit_client(client_p, client_p, client_p, "Missing required CAPABs");
 
-		if (!(client_p->localClient->caps & cap->cap))
-		{
-			char exitbuf[BUFSIZE];
-
-			rb_snprintf(exitbuf, BUFSIZE, "Missing required CAPAB [%s]", cap->name);
-			exit_client(client_p, client_p, client_p, exitbuf);
-
-			return 0;
-		}
+		return 0;
 	}
 
 	/* Now we just have to call check_server and everything should be
@@ -151,14 +144,14 @@ mr_server(struct Client *client_p, struct Client *source_p, int parc, const char
 	case -2:
 		sendto_realops_snomask(SNO_GENERAL, is_remote_connect(client_p) ? L_NETWIDE : L_ALL,
 				     "Unauthorised server connection attempt from %s: "
-				     "Bad password for server %s",
+				     "Bad credentials for server %s",
 				     "[@255.255.255.255]", name);
 
-		ilog(L_SERVER, "Access denied, invalid password for server %s%s",
+		ilog(L_SERVER, "Access denied, invalid credentials for server %s%s",
 		     EmptyString(client_p->name) ? name : "",
 		     log_client_name(client_p, SHOW_IP));
 
-		exit_client(client_p, client_p, client_p, "Invalid password.");
+		exit_client(client_p, client_p, client_p, "Invalid credentials.");
 		return 0;
 		/* NOT REACHED */
 		break;
@@ -492,13 +485,10 @@ ms_sid(struct Client *client_p, struct Client *source_p, int parc, const char *p
 	struct remote_conf *hub_p;
 	hook_data_client hdata;
 	rb_dlink_node *ptr;
-	int hop;
 	int hlined = 0;
 	int llined = 0;
 	char squitreason[160];
 
-	hop = atoi(parv[2]);
-
 	/* collision on the name? */
 	if((target_p = find_server(NULL, parv[1])) != NULL)
 	{
