@@ -219,12 +219,6 @@ parse_resv(struct Client *source_p, const char *name, const char *reason, int te
 			return;
 		}
 
-		if(strchr(reason, '"'))
-		{
-			sendto_one_notice(source_p, ":Invalid character '\"' in comment");
-			return;
-		}
-
 		aconf = make_conf();
 		aconf->status = CONF_RESV_CHANNEL;
 		aconf->port = 0;
@@ -293,12 +287,6 @@ parse_resv(struct Client *source_p, const char *name, const char *reason, int te
 			return;
 		}
 
-		if(strchr(reason, '"'))
-		{
-			sendto_one_notice(source_p, ":Invalid character '\"' in comment");
-			return;
-		}
-
 		if(!valid_wild_card_simple(name))
 		{
 			sendto_one_notice(source_p,
@@ -373,6 +361,7 @@ parse_resv(struct Client *source_p, const char *name, const char *reason, int te
 		}
 
 		rb_dlinkAddAlloc(aconf, &resv_conf_list);
+		resv_nick_fnc(aconf->host, aconf->passwd, temp_time);
 	}
 	else
 		sendto_one_notice(source_p, ":You have specified an invalid resv: [%s]", name);
@@ -530,7 +519,7 @@ remove_resv(struct Client *source_p, const char *name, int propagated)
 		if((aconf = hash_find_resv(name)) == NULL)
 		{
 			if(propagated && rb_dlink_list_length(&cluster_conf_list))
-				cluster_generic(source_p, "UNXLINE", SHARED_UNXLINE, CAP_CLUSTER, "%s", name);
+				cluster_generic(source_p, "UNRESV", SHARED_UNRESV, CAP_CLUSTER, "%s", name);
 
 			sendto_one_notice(source_p, ":No RESV for %s", name);
 			return;
@@ -603,7 +592,7 @@ remove_resv(struct Client *source_p, const char *name, int propagated)
 		if(aconf == NULL)
 		{
 			if(propagated && rb_dlink_list_length(&cluster_conf_list))
-				cluster_generic(source_p, "UNXLINE", SHARED_UNXLINE, CAP_CLUSTER, "%s", name);
+				cluster_generic(source_p, "UNRESV", SHARED_UNRESV, CAP_CLUSTER, "%s", name);
 
 			sendto_one_notice(source_p, ":No RESV for %s", name);
 			return;
