@@ -120,6 +120,8 @@ static void stats_servlinks(struct Client *);
 static void stats_ltrace(struct Client *, int, const char **);
 static void stats_ziplinks(struct Client *);
 static void stats_comm(struct Client *);
+static void stats_capability(struct Client *);
+
 /* This table contains the possible stats items, in order:
  * stats letter,  function to call, operonly? adminonly?
  * case only matters in the stats letter column.. -- fl_
@@ -131,7 +133,7 @@ static struct StatsStruct stats_cmd_table[] = {
 	{'b', stats_delay,		1, 1, },
 	{'B', stats_hash,		1, 1, },
 	{'c', stats_connect,		0, 0, },
-	{'C', stats_connect,		0, 0, },
+	{'C', stats_capability,		1, 0, },
 	{'d', stats_tdeny,		1, 0, },
 	{'D', stats_deny,		1, 0, },
 	{'e', stats_exempt,		1, 0, },
@@ -706,6 +708,20 @@ stats_oper(struct Client *source_p)
 	}
 }
 
+static void
+stats_capability_walk(const char *line, void *data)
+{
+	struct Client *client_p = data;
+
+	sendto_one_numeric(client_p, RPL_STATSDEBUG, "C :%s", line);
+}
+
+static void
+stats_capability(struct Client *client_p)
+{
+	capability_index_stats(stats_capability_walk, client_p);
+}
+
 static void
 stats_privset(struct Client *source_p)
 {
@@ -928,6 +944,8 @@ stats_tstats (struct Client *source_p)
 	sendto_one_numeric(source_p, RPL_STATSDEBUG,
 			   "T :tgchange blocked msgs %u restricted addrs %lu",
 			   sp.is_tgch, rb_dlink_list_length(&tgchange_list));
+	sendto_one_numeric(source_p, RPL_STATSDEBUG,
+			   "T :ratelimit blocked commands %u", sp.is_rl);
 	sendto_one_numeric(source_p, RPL_STATSDEBUG,
 			   "T :auth successes %u fails %u",
 			   sp.is_asuc, sp.is_abad);
@@ -1147,7 +1165,6 @@ stats_memory (struct Client *source_p)
 {
 	struct Client *target_p;
 	struct Channel *chptr;
-	struct Ban *actualBan;
 	rb_dlink_node *rb_dlink;
 	rb_dlink_node *ptr;
 	int channel_count = 0;
@@ -1229,7 +1246,6 @@ stats_memory (struct Client *source_p)
 
 		RB_DLINK_FOREACH(rb_dlink, chptr->banlist.head)
 		{
-			actualBan = rb_dlink->data;
 			channel_bans++;
 
 			channel_ban_memory += sizeof(rb_dlink_node) + sizeof(struct Ban);
@@ -1237,7 +1253,6 @@ stats_memory (struct Client *source_p)
 
 		RB_DLINK_FOREACH(rb_dlink, chptr->exceptlist.head)
 		{
-			actualBan = rb_dlink->data;
 			channel_except++;
 
 			channel_except_memory += (sizeof(rb_dlink_node) + sizeof(struct Ban));
@@ -1245,7 +1260,6 @@ stats_memory (struct Client *source_p)
 
 		RB_DLINK_FOREACH(rb_dlink, chptr->invexlist.head)
 		{
-			actualBan = rb_dlink->data;
 			channel_invex++;
 
 			channel_invex_memory += (sizeof(rb_dlink_node) + sizeof(struct Ban));
@@ -1253,7 +1267,6 @@ stats_memory (struct Client *source_p)
 
 		RB_DLINK_FOREACH(rb_dlink, chptr->quietlist.head)
 		{
-			actualBan = rb_dlink->data;
 			channel_quiets++;
 
 			channel_quiet_memory += (sizeof(rb_dlink_node) + sizeof(struct Ban));
