@@ -65,12 +65,15 @@ static int mode_count;
 static int mode_limit;
 static int mode_limit_simple;
 static int mask_pos;
+static int removed_mask_pos;
 
 char cflagsbuf[256];
 char cflagsmyinfo[256];
 
 int chmode_flags[256];
 
+extern int h_get_channel_access;
+
 /* OPTIMIZE ME! -- dwr */
 void
 construct_cflags_strings(void)
@@ -101,20 +104,10 @@ construct_cflags_strings(void)
                 
 		switch (chmode_flags[i])
 		{
-		    case MODE_EXLIMIT:
+		    case MODE_FREETARGET:
 		    case MODE_DISFORWARD:
 			if(ConfigChannel.use_forward)
-			{
-			    *ptr++ = (char) i;
-			}
-			
-			break;
-		    case MODE_REGONLY:
-			if(rb_dlink_list_length(&service_list))
-			{
-			    *ptr++ = (char) i;
-			}
-
+				*ptr++ = (char) i;
 			break;
 		    default:
 			if(chmode_flags[i] != 0)
@@ -184,23 +177,72 @@ cflag_orphan(char c_)
 	construct_cflags_strings();
 }
 
-static int
+int
 get_channel_access(struct Client *source_p, struct membership *msptr)
 {
-	if(!MyClient(source_p) || is_chanop(msptr))
+	hook_data_channel_approval moduledata;
+
+	if(!MyClient(source_p))
 		return CHFL_CHANOP;
 
-	return CHFL_PEON;
+	if (msptr == NULL)
+		return CHFL_PEON;
+
+	moduledata.client = source_p;
+	moduledata.chptr = msptr->chptr;
+	moduledata.msptr = msptr;
+	moduledata.target = NULL;
+	moduledata.approved = is_chanop(msptr) ? CHFL_CHANOP : CHFL_PEON;
+
+	call_hook(h_get_channel_access, &moduledata);
+
+	return moduledata.approved;
+}
+
+/* allow_mode_change()
+ *
+ * Checks if mlock and chanops permit a mode change.
+ *
+ * inputs	- client, channel, access level, errors pointer, mode char
+ * outputs	- 0 on failure, 1 on success
+ * side effects - error message sent on failure
+ */
+static int
+allow_mode_change(struct Client *source_p, struct Channel *chptr, int alevel,
+		int *errors, char c)
+{
+	/* If this mode char is locked, don't allow local users to change it. */
+	if (MyClient(source_p) && chptr->mode_lock && strchr(chptr->mode_lock, c))
+	{
+		if (!(*errors & SM_ERR_MLOCK))
+			sendto_one_numeric(source_p,
+					ERR_MLOCKRESTRICTED,
+					form_str(ERR_MLOCKRESTRICTED),
+					chptr->chname,
+					c,
+					chptr->mode_lock);
+		*errors |= SM_ERR_MLOCK;
+		return 0;
+	}
+	if(alevel != CHFL_CHANOP)
+	{
+		if(!(*errors & SM_ERR_NOOPS))
+			sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED),
+				   me.name, source_p->name, chptr->chname);
+		*errors |= SM_ERR_NOOPS;
+		return 0;
+	}
+	return 1;
 }
 
 /* add_id()
  *
- * inputs	- client, channel, id to add, type
+ * inputs	- client, channel, id to add, type, forward
  * outputs	- 0 on failure, 1 on success
  * side effects - given id is added to the appropriate list
  */
 int
-add_id(struct Client *source_p, struct Channel *chptr, const char *banid,
+add_id(struct Client *source_p, struct Channel *chptr, const char *banid, const char *forward,
        rb_dlink_list * list, long mode_type)
 {
 	struct Ban *actualBan;
@@ -244,7 +286,7 @@ add_id(struct Client *source_p, struct Channel *chptr, const char *banid,
 	else
 		rb_strlcpy(who, source_p->name, sizeof(who));
 
-	actualBan = allocate_ban(realban, who);
+	actualBan = allocate_ban(realban, who, forward);
 	actualBan->when = rb_current_time();
 
 	rb_dlinkAdd(actualBan, &actualBan->node, list);
@@ -259,17 +301,17 @@ add_id(struct Client *source_p, struct Channel *chptr, const char *banid,
 /* del_id()
  *
  * inputs	- channel, id to remove, type
- * outputs	- 0 on failure, 1 on success
- * side effects - given id is removed from the appropriate list
+ * outputs	- pointer to ban that was removed, if any
+ * side effects - given id is removed from the appropriate list and returned
  */
-int
+struct Ban *
 del_id(struct Channel *chptr, const char *banid, rb_dlink_list * list, long mode_type)
 {
 	rb_dlink_node *ptr;
 	struct Ban *banptr;
 
 	if(EmptyString(banid))
-		return 0;
+		return NULL;
 
 	RB_DLINK_FOREACH(ptr, list->head)
 	{
@@ -278,17 +320,16 @@ del_id(struct Channel *chptr, const char *banid, rb_dlink_list * list, long mode
 		if(irccmp(banid, banptr->banstr) == 0)
 		{
 			rb_dlinkDelete(&banptr->node, list);
-			free_ban(banptr);
 
 			/* invalidate the can_send() cache */
 			if(mode_type == CHFL_BAN || mode_type == CHFL_QUIET || mode_type == CHFL_EXCEPTION)
 				chptr->bants++;
 
-			return 1;
+			return banptr;
 		}
 	}
 
-	return 0;
+	return NULL;
 }
 
 /* check_string()
@@ -332,10 +373,11 @@ pretty_mask(const char *idmask)
 {
 	static char mask_buf[BUFSIZE];
 	int old_mask_pos;
-	char *nick, *user, *host;
+	char *nick, *user, *host, *forward = NULL;
 	char splat[] = "*";
-	char *t, *at, *ex;
-	char ne = 0, ue = 0, he = 0;	/* save values at nick[NICKLEN], et all */
+	char *t, *at, *ex, *ex2;
+	char ne = 0, ue = 0, he = 0, fe = 0;	/* save values at nick[NICKLEN], et all */
+	char e2 = 0;				/* save value that delimits forward channel */
 	char *mask;
 
 	mask = LOCAL_COPY(idmask);
@@ -361,7 +403,7 @@ pretty_mask(const char *idmask)
 		return mask_buf + old_mask_pos;
 	}
 
-	at = ex = NULL;
+	at = ex = ex2 = NULL;
 	if((t = strchr(mask, '@')) != NULL)
 	{
 		at = t;
@@ -383,6 +425,15 @@ pretty_mask(const char *idmask)
 			if(*mask != '\0')
 				user = mask;
 		}
+
+		if((t = strchr(host, '!')) != NULL || (t = strchr(host, '$')) != NULL)
+		{
+			ex2 = t;
+			e2 = *t;
+			*t++= '\0';
+			if (*t != '\0')
+				forward = t;
+		}
 	}
 	else if((t = strchr(mask, '!')) != NULL)
 	{
@@ -393,7 +444,7 @@ pretty_mask(const char *idmask)
 		if(*t != '\0')
 			user = t;
 	}
-	else if(strchr(mask, '.') != NULL || strchr(mask, ':') != NULL || strchr(mask, '/') != NULL)
+	else if(strchr(mask, '.') != NULL || strchr(mask, ':') != NULL)
 	{
 		if(*mask != '\0')
 			host = mask;
@@ -420,24 +471,81 @@ pretty_mask(const char *idmask)
 		he = host[HOSTLEN];
 		host[HOSTLEN] = '\0';
 	}
+	if(forward && strlen(forward) > CHANNELLEN)
+	{
+		fe = forward[CHANNELLEN];
+		forward[CHANNELLEN] = '\0';
+	}
 
-	mask_pos += rb_sprintf(mask_buf + mask_pos, "%s!%s@%s", nick, user, host) + 1;
+	if (forward)
+		mask_pos += rb_sprintf(mask_buf + mask_pos, "%s!%s@%s$%s", nick, user, host, forward) + 1;
+	else
+		mask_pos += rb_sprintf(mask_buf + mask_pos, "%s!%s@%s", nick, user, host) + 1;
 
 	/* restore mask, since we may need to use it again later */
 	if(at)
 		*at = '@';
 	if(ex)
 		*ex = '!';
+	if(ex2)
+		*ex2 = e2;
 	if(ne)
 		nick[NICKLEN - 1] = ne;
 	if(ue)
 		user[USERLEN] = ue;
 	if(he)
 		host[HOSTLEN] = he;
+	if(fe)
+		forward[CHANNELLEN] = fe;
 
 	return mask_buf + old_mask_pos;
 }
 
+/* check_forward()
+ *
+ * input	- client, channel to set mode on, target channel name
+ * output	- true if forwarding should be allowed
+ * side effects - numeric sent if not allowed
+ */
+static int
+check_forward(struct Client *source_p, struct Channel *chptr,
+		const char *forward)
+{
+	struct Channel *targptr;
+	struct membership *msptr;
+
+	if(!check_channel_name(forward) ||
+			(MyClient(source_p) && (strlen(forward) > LOC_CHANNELLEN || hash_find_resv(forward))))
+	{
+		sendto_one_numeric(source_p, ERR_BADCHANNAME, form_str(ERR_BADCHANNAME), forward);
+		return 0;
+	}
+	/* don't forward to inconsistent target -- jilles */
+	if(chptr->chname[0] == '#' && forward[0] == '&')
+	{
+		sendto_one_numeric(source_p, ERR_BADCHANNAME,
+				   form_str(ERR_BADCHANNAME), forward);
+		return 0;
+	}
+	if(MyClient(source_p) && (targptr = find_channel(forward)) == NULL)
+	{
+		sendto_one_numeric(source_p, ERR_NOSUCHCHANNEL,
+				   form_str(ERR_NOSUCHCHANNEL), forward);
+		return 0;
+	}
+	if(MyClient(source_p) && !(targptr->mode.mode & MODE_FREETARGET))
+	{
+		if((msptr = find_channel_membership(targptr, source_p)) == NULL ||
+			get_channel_access(source_p, msptr) != CHFL_CHANOP)
+		{
+			sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED),
+				   me.name, source_p->name, targptr->chname);
+			return 0;
+		}
+	}
+	return 1;
+}
+
 /* fix_key()
  *
  * input	- key to fix
@@ -504,14 +612,8 @@ chm_simple(struct Client *source_p, struct Channel *chptr,
 	   int alevel, int parc, int *parn,
 	   const char **parv, int *errors, int dir, char c, long mode_type)
 {
-	if(alevel != CHFL_CHANOP)
-	{
-		if(!(*errors & SM_ERR_NOOPS))
-			sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED),
-				   me.name, source_p->name, chptr->chname);
-		*errors |= SM_ERR_NOOPS;
+	if(!allow_mode_change(source_p, chptr, alevel, errors, c))
 		return;
-	}
 
 	if(MyClient(source_p) && (++mode_limit_simple > MAXMODES_SIMPLE))
 		return;
@@ -521,7 +623,7 @@ chm_simple(struct Client *source_p, struct Channel *chptr,
 	{
 		/* if +f is disabled, ignore an attempt to set +QF locally */
 		if(!ConfigChannel.use_forward && MyClient(source_p) &&
-		   (c == 'Q' || c == 'F'))
+				(c == 'Q' || c == 'F'))
 			return;
 
 		chptr->mode.mode |= mode_type;
@@ -638,8 +740,7 @@ chm_ban(struct Client *source_p, struct Channel *chptr,
 	int alevel, int parc, int *parn,
 	const char **parv, int *errors, int dir, char c, long mode_type)
 {
-	const char *mask;
-	const char *raw_mask;
+	char *mask, *raw_mask, *forward;
 	rb_dlink_list *list;
 	rb_dlink_node *ptr;
 	struct Ban *banptr;
@@ -718,6 +819,7 @@ chm_ban(struct Client *source_p, struct Channel *chptr,
 		*errors |= errorval;
 
 		/* non-ops cant see +eI lists.. */
+		/* note that this is still permitted if +e/+I are mlocked. */
 		if(alevel != CHFL_CHANOP && mode_type != CHFL_BAN &&
 				mode_type != CHFL_QUIET)
 		{
@@ -730,23 +832,24 @@ chm_ban(struct Client *source_p, struct Channel *chptr,
 
 		RB_DLINK_FOREACH(ptr, list->head)
 		{
+			char buf[BANLEN];
 			banptr = ptr->data;
+			if(banptr->forward)
+				rb_snprintf(buf, sizeof(buf), "%s$%s", banptr->banstr, banptr->forward);
+			else
+				rb_strlcpy(buf, banptr->banstr, sizeof(buf));
+
 			sendto_one(source_p, form_str(rpl_list),
 				   me.name, source_p->name, chptr->chname,
-				   banptr->banstr, banptr->who, banptr->when);
+				   buf, banptr->who, banptr->when);
 		}
 		sendto_one(source_p, form_str(rpl_endlist), me.name, source_p->name, chptr->chname);
 		return;
 	}
 
-	if(alevel != CHFL_CHANOP)
-	{
-		if(!(*errors & SM_ERR_NOOPS))
-			sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED),
-				   me.name, source_p->name, chptr->chname);
-		*errors |= SM_ERR_NOOPS;
+	if(!allow_mode_change(source_p, chptr, alevel, errors, c))
 		return;
-	}
+
 
 	if(MyClient(source_p) && (++mode_limit > MAXMODEPARAMS))
 		return;
@@ -773,7 +876,23 @@ chm_ban(struct Client *source_p, struct Channel *chptr,
 	 * name etc.
 	 */
 	if(strlen(mask) > IRCD_MIN(BANLEN, MODEBUFLEN - 5))
+	{
+		sendto_one_numeric(source_p, ERR_INVALIDBAN,
+				form_str(ERR_INVALIDBAN),
+				chptr->chname, c, raw_mask);
 		return;
+	}
+
+	/* Look for a $ after the first character.
+	 * As the first character, it marks an extban; afterwards
+	 * it delimits a forward channel.
+	 */
+	if ((forward = strchr(mask+1, '$')) != NULL)
+	{
+		*forward++ = '\0';
+		if (*forward == '\0')
+			forward = NULL;
+	}
 
 	/* if we're adding a NEW id */
 	if(dir == MODE_ADD)
@@ -781,16 +900,52 @@ chm_ban(struct Client *source_p, struct Channel *chptr,
 		if (*mask == '$' && MyClient(source_p))
 		{
 			if (!valid_extban(mask, source_p, chptr, mode_type))
-				/* XXX perhaps return an error message here */
+			{
+				sendto_one_numeric(source_p, ERR_INVALIDBAN,
+						form_str(ERR_INVALIDBAN),
+						chptr->chname, c, raw_mask);
 				return;
+			}
+		}
+
+		/* For compatibility, only check the forward channel from
+		 * local clients. Accept any forward channel from servers.
+		 */
+		if(forward != NULL && MyClient(source_p))
+		{
+			/* For simplicity and future flexibility, do not
+			 * allow '$' in forwarding targets.
+			 */
+			if(!ConfigChannel.use_forward ||
+					strchr(forward, '$') != NULL)
+			{
+				sendto_one_numeric(source_p, ERR_INVALIDBAN,
+						form_str(ERR_INVALIDBAN),
+						chptr->chname, c, raw_mask);
+				return;
+			}
+			/* check_forward() sends its own error message */
+			if(!check_forward(source_p, chptr, forward))
+				return;
+			/* Forwards only make sense for bans. */
+			if(mode_type != CHFL_BAN)
+			{
+				sendto_one_numeric(source_p, ERR_INVALIDBAN,
+						form_str(ERR_INVALIDBAN),
+						chptr->chname, c, raw_mask);
+				return;
+			}
 		}
 
 		/* dont allow local clients to overflow the banlist, dont
 		 * let remote servers set duplicate bans
 		 */
-		if(!add_id(source_p, chptr, mask, list, mode_type))
+		if(!add_id(source_p, chptr, mask, forward, list, mode_type))
 			return;
 
+		if(forward)
+			forward[-1]= '$';
+
 		mode_changes[mode_count].letter = c;
 		mode_changes[mode_count].dir = MODE_ADD;
 		mode_changes[mode_count].caps = caps;
@@ -801,20 +956,33 @@ chm_ban(struct Client *source_p, struct Channel *chptr,
 	}
 	else if(dir == MODE_DEL)
 	{
-		if(del_id(chptr, mask, list, mode_type) == 0)
+		struct Ban *removed;
+		static char buf[BANLEN * MAXMODEPARAMS];
+		int old_removed_mask_pos = removed_mask_pos;
+		if((removed = del_id(chptr, mask, list, mode_type)) == NULL)
 		{
 			/* mask isn't a valid ban, check raw_mask */
-			if(del_id(chptr, raw_mask, list, mode_type))
+			if((removed = del_id(chptr, raw_mask, list, mode_type)) != NULL)
 				mask = raw_mask;
 		}
 
+		if(removed && removed->forward)
+			removed_mask_pos += rb_snprintf(buf + old_removed_mask_pos, sizeof(buf), "%s$%s", removed->banstr, removed->forward) + 1;
+		else
+			removed_mask_pos += rb_strlcpy(buf + old_removed_mask_pos, mask, sizeof(buf)) + 1;
+		if(removed)
+		{
+			free_ban(removed);
+			removed = NULL;
+		}
+
 		mode_changes[mode_count].letter = c;
 		mode_changes[mode_count].dir = MODE_DEL;
 		mode_changes[mode_count].caps = caps;
 		mode_changes[mode_count].nocaps = 0;
 		mode_changes[mode_count].mems = mems;
 		mode_changes[mode_count].id = NULL;
-		mode_changes[mode_count++].arg = mask;
+		mode_changes[mode_count++].arg = buf + old_removed_mask_pos;
 	}
 }
 
@@ -827,14 +995,8 @@ chm_op(struct Client *source_p, struct Channel *chptr,
 	const char *opnick;
 	struct Client *targ_p;
 
-	if(alevel != CHFL_CHANOP)
-	{
-		if(!(*errors & SM_ERR_NOOPS))
-			sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED),
-				   me.name, source_p->name, chptr->chname);
-		*errors |= SM_ERR_NOOPS;
+	if(!allow_mode_change(source_p, chptr, alevel, errors, c))
 		return;
-	}
 
 	if((dir == MODE_QUERY) || (parc <= *parn))
 		return;
@@ -870,7 +1032,7 @@ chm_op(struct Client *source_p, struct Channel *chptr,
 
 	if(dir == MODE_ADD)
 	{
-		if(targ_p == source_p)
+		if(targ_p == source_p && mstptr->flags & CHFL_CHANOP)
 			return;
 
 		mode_changes[mode_count].letter = c;
@@ -915,14 +1077,8 @@ chm_voice(struct Client *source_p, struct Channel *chptr,
 	const char *opnick;
 	struct Client *targ_p;
 
-	if(alevel != CHFL_CHANOP)
-	{
-		if(!(*errors & SM_ERR_NOOPS))
-			sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED),
-				   me.name, source_p->name, chptr->chname);
-		*errors |= SM_ERR_NOOPS;
+	if(!allow_mode_change(source_p, chptr, alevel, errors, c))
 		return;
-	}
 
 	if((dir == MODE_QUERY) || parc <= *parn)
 		return;
@@ -993,14 +1149,8 @@ chm_limit(struct Client *source_p, struct Channel *chptr,
 	static char limitstr[30];
 	int limit;
 
-	if(alevel != CHFL_CHANOP)
-	{
-		if(!(*errors & SM_ERR_NOOPS))
-			sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED),
-				   me.name, source_p->name, chptr->chname);
-		*errors |= SM_ERR_NOOPS;
+	if(!allow_mode_change(source_p, chptr, alevel, errors, c))
 		return;
-	}
 
 	if(dir == MODE_QUERY)
 		return;
@@ -1052,14 +1202,8 @@ chm_throttle(struct Client *source_p, struct Channel *chptr,
 {
 	int joins = 0, timeslice = 0;
 
-	if(alevel != CHFL_CHANOP)
-	{
-		if(!(*errors & SM_ERR_NOOPS))
-			sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED),
-				   me.name, source_p->name, chptr->chname);
-		*errors |= SM_ERR_NOOPS;
+	if(!allow_mode_change(source_p, chptr, alevel, errors, c))
 		return;
-	}
 
 	if(dir == MODE_QUERY)
 		return;
@@ -1112,8 +1256,6 @@ chm_forward(struct Client *source_p, struct Channel *chptr,
 	int alevel, int parc, int *parn,
 	const char **parv, int *errors, int dir, char c, long mode_type)
 {
-	struct Channel *targptr = NULL;
-	struct membership *msptr;
 	const char *forward;
 
 	/* if +f is disabled, ignore local attempts to set it */
@@ -1135,14 +1277,8 @@ chm_forward(struct Client *source_p, struct Channel *chptr,
 	}
 
 #ifndef FORWARD_OPERONLY
-	if(alevel != CHFL_CHANOP)
-	{
-		if(!(*errors & SM_ERR_NOOPS))
-			sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED),
-				   me.name, source_p->name, chptr->chname);
-		*errors |= SM_ERR_NOOPS;
+	if(!allow_mode_change(source_p, chptr, alevel, errors, c))
 		return;
-	}
 #else
 	if(!IsOper(source_p) && !IsServer(source_p))
 	{
@@ -1163,35 +1299,9 @@ chm_forward(struct Client *source_p, struct Channel *chptr,
 
 		if(EmptyString(forward))
 			return;
-		if(!check_channel_name(forward) ||
-				(MyClient(source_p) && (strlen(forward) > LOC_CHANNELLEN || hash_find_resv(forward))))
-		{
-			sendto_one_numeric(source_p, ERR_BADCHANNAME, form_str(ERR_BADCHANNAME), forward);
+
+		if(!check_forward(source_p, chptr, forward))
 			return;
-		}
-		/* don't forward to inconsistent target -- jilles */
-		if(chptr->chname[0] == '#' && forward[0] == '&')
-		{
-			sendto_one_numeric(source_p, ERR_BADCHANNAME,
-					   form_str(ERR_BADCHANNAME), forward);
-			return;
-		}
-		if(MyClient(source_p) && (targptr = find_channel(forward)) == NULL)
-		{
-			sendto_one_numeric(source_p, ERR_NOSUCHCHANNEL,
-					   form_str(ERR_NOSUCHCHANNEL), forward);
-			return;
-		}
-		if(MyClient(source_p) && !(targptr->mode.mode & MODE_FREETARGET))
-		{
-			if((msptr = find_channel_membership(targptr, source_p)) == NULL ||
-				get_channel_access(source_p, msptr) != CHFL_CHANOP)
-			{
-				sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED),
-					   me.name, source_p->name, targptr->chname);
-				return;
-			}
-		}
 
 		rb_strlcpy(chptr->mode.forward, forward, sizeof(chptr->mode.forward));
 
@@ -1199,7 +1309,8 @@ chm_forward(struct Client *source_p, struct Channel *chptr,
 		mode_changes[mode_count].dir = MODE_ADD;
 		mode_changes[mode_count].caps = 0;
 		mode_changes[mode_count].nocaps = 0;
-		mode_changes[mode_count].mems = ConfigChannel.use_forward ? ALL_MEMBERS : ONLY_SERVERS;
+		mode_changes[mode_count].mems =
+			ConfigChannel.use_forward ? ALL_MEMBERS : ONLY_SERVERS;
 		mode_changes[mode_count].id = NULL;
 		mode_changes[mode_count++].arg = forward;
 	}
@@ -1227,14 +1338,8 @@ chm_key(struct Client *source_p, struct Channel *chptr,
 {
 	char *key;
 
-	if(alevel != CHFL_CHANOP)
-	{
-		if(!(*errors & SM_ERR_NOOPS))
-			sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED),
-				   me.name, source_p->name, chptr->chname);
-		*errors |= SM_ERR_NOOPS;
+	if(!allow_mode_change(source_p, chptr, alevel, errors, c))
 		return;
-	}
 
 	if(dir == MODE_QUERY)
 		return;
@@ -1299,44 +1404,6 @@ chm_key(struct Client *source_p, struct Channel *chptr,
 	}
 }
 
-void
-chm_regonly(struct Client *source_p, struct Channel *chptr,
-	    int alevel, int parc, int *parn,
-	    const char **parv, int *errors, int dir, char c, long mode_type)
-{
-	if(alevel != CHFL_CHANOP)
-	{
-		if(!(*errors & SM_ERR_NOOPS))
-			sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED),
-				   me.name, source_p->name, chptr->chname);
-		*errors |= SM_ERR_NOOPS;
-		return;
-	}
-
-	if(dir == MODE_QUERY)
-		return;
-
-	if(((dir == MODE_ADD) && (chptr->mode.mode & mode_type)) ||
-	   ((dir == MODE_DEL) && !(chptr->mode.mode & mode_type)))
-		return;
-
-	if(MyClient(source_p) && (++mode_limit_simple > MAXMODES_SIMPLE))
-		return;
-
-	if(dir == MODE_ADD)
-		chptr->mode.mode |= mode_type;
-	else
-		chptr->mode.mode &= ~mode_type;
-
-	mode_changes[mode_count].letter = c;
-	mode_changes[mode_count].dir = dir;
-	mode_changes[mode_count].caps = CAP_SERVICE;
-	mode_changes[mode_count].nocaps = 0;
-	mode_changes[mode_count].mems = ALL_MEMBERS;
-	mode_changes[mode_count].id = NULL;
-	mode_changes[mode_count++].arg = NULL;
-}
-
 /* *INDENT-OFF* */
 struct ChannelMode chmode_table[256] =
 {
@@ -1455,7 +1522,7 @@ struct ChannelMode chmode_table[256] =
   {chm_op,	0 },			/* o */
   {chm_simple,	MODE_PRIVATE },		/* p */
   {chm_ban,	CHFL_QUIET },		/* q */
-  {chm_regonly, MODE_REGONLY },		/* r */
+  {chm_simple,  MODE_REGONLY },		/* r */
   {chm_simple,	MODE_SECRET },		/* s */
   {chm_simple,	MODE_TOPICLIMIT },	/* t */
   {chm_nosuch,	0 },			/* u */
@@ -1637,6 +1704,7 @@ set_channel_mode(struct Client *client_p, struct Client *source_p,
 	struct Client *fakesource_p;
 
 	mask_pos = 0;
+	removed_mask_pos = 0;
 	mode_count = 0;
 	mode_limit = 0;
 	mode_limit_simple = 0;
@@ -1663,19 +1731,6 @@ set_channel_mode(struct Client *client_p, struct Client *source_p,
 			dir = MODE_QUERY;
 			break;
 		default:
-			/* If this mode char is locked, don't allow local users to change it. */
-			if (MyClient(source_p) && chptr->mode_lock && strchr(chptr->mode_lock, c))
-			{
-				if (!(errors & SM_ERR_MLOCK))
-					sendto_one_numeric(source_p,
-							ERR_MLOCKRESTRICTED,
-							form_str(ERR_MLOCKRESTRICTED),
-							chptr->chname,
-							c,
-							chptr->mode_lock);
-				errors |= SM_ERR_MLOCK;
-				continue;
-			}
 			chmode_table[(unsigned char) c].set_func(fakesource_p, chptr, alevel,
 				       parc, &parn, parv,
 				       &errors, dir, c,
