@@ -32,6 +32,8 @@
 
 #define CF_TYPE(x) ((x) & CF_MTYPE)
 
+static int yy_defer_accept = 1;
+
 struct TopConf *conf_cur_block;
 static char *conf_cur_block_name;
 
@@ -54,6 +56,9 @@ static struct alias_entry *yy_alias = NULL;
 
 static char *yy_blacklist_host = NULL;
 static char *yy_blacklist_reason = NULL;
+static int yy_blacklist_ipv4 = 1;
+static int yy_blacklist_ipv6 = 0;
+
 static char *yy_privset_extends = NULL;
 
 static const char *
@@ -257,6 +262,25 @@ conf_set_serverinfo_vhost6(void *data)
 #endif
 }
 
+static void
+conf_set_serverinfo_nicklen(void *data)
+{
+	ConfigFileEntry.nicklen = (*(unsigned int *) data) + 1;
+
+	if (ConfigFileEntry.nicklen > NICKLEN)
+	{
+		conf_report_error("Warning -- ignoring serverinfo::nicklen -- provided nicklen (%u) is greater than allowed nicklen (%u)",
+				  ConfigFileEntry.nicklen - 1, NICKLEN - 1);
+		ConfigFileEntry.nicklen = NICKLEN;
+	}
+	else if (ConfigFileEntry.nicklen < 9 + 1)
+	{
+		conf_report_error("Warning -- serverinfo::nicklen is too low (%u) -- forcing 9",
+				  ConfigFileEntry.nicklen);
+		ConfigFileEntry.nicklen = 9 + 1;
+	}
+}
+
 static void
 conf_set_modules_module(void *data)
 {
@@ -595,6 +619,9 @@ conf_end_oper(struct TopConf *tc)
 				return 0;
 			}
 		}
+
+		if(!EmptyString(yy_oper->certfp))
+			yy_tmpoper->certfp = rb_strdup(yy_oper->certfp);
 #endif
 
 		/* all is ok, put it on oper_conf_list */
@@ -618,6 +645,8 @@ conf_set_oper_flags(void *data)
 static void
 conf_set_oper_fingerprint(void *data)
 {
+	if (yy_oper->certfp)
+		rb_free(yy_oper->certfp);
 	yy_oper->certfp = rb_strdup((char *) data);
 }
 
@@ -817,7 +846,11 @@ conf_end_listen(struct TopConf *tc)
 	return 0;
 }
 
-
+static void
+conf_set_listen_defer_accept(void *data)
+{
+	yy_defer_accept = *(unsigned int *) data;
+}
 
 static void
 conf_set_listen_port_both(void *data, int ssl)
@@ -833,9 +866,9 @@ conf_set_listen_port_both(void *data, int ssl)
 		}
                 if(listener_address == NULL)
                 {
-			add_listener(args->v.number, listener_address, AF_INET, ssl);
+			add_listener(args->v.number, listener_address, AF_INET, ssl, yy_defer_accept);
 #ifdef RB_IPV6
-			add_listener(args->v.number, listener_address, AF_INET6, ssl);
+			add_listener(args->v.number, listener_address, AF_INET6, ssl, yy_defer_accept);
 #endif
                 }
 		else
@@ -848,7 +881,7 @@ conf_set_listen_port_both(void *data, int ssl)
 #endif
 				family = AF_INET;
 		
-			add_listener(args->v.number, listener_address, family, ssl);
+			add_listener(args->v.number, listener_address, family, ssl, yy_defer_accept);
                 
                 }
 
@@ -1242,9 +1275,9 @@ conf_end_connect(struct TopConf *tc)
 		return 0;
 	}
 
-	if(EmptyString(yy_server->passwd) || EmptyString(yy_server->spasswd))
+	if((EmptyString(yy_server->passwd) || EmptyString(yy_server->spasswd)) && EmptyString(yy_server->certfp))
 	{
-		conf_report_error("Ignoring connect block for %s -- missing password.",
+		conf_report_error("Ignoring connect block for %s -- no fingerprint or password credentials provided.",
 					yy_server->name);
 		return 0;
 	}
@@ -1316,6 +1349,17 @@ conf_set_connect_accept_password(void *data)
 	yy_server->passwd = rb_strdup(data);
 }
 
+static void
+conf_set_connect_fingerprint(void *data)
+{
+	if (yy_server->certfp)
+		rb_free(yy_server->certfp);
+	yy_server->certfp = rb_strdup((char *) data);
+
+	/* force SSL to be enabled if fingerprint is enabled. */
+	yy_server->flags |= SERVER_SSL;
+}
+
 static void
 conf_set_connect_port(void *data)
 {
@@ -1574,6 +1618,7 @@ conf_set_general_default_umodes(void *data)
 		/* don't allow +o */
 		case 'o':
 		case 'S':
+		case 'Z':
 		case ' ':
 			break;
 
@@ -1759,6 +1804,34 @@ conf_set_blacklist_host(void *data)
 	yy_blacklist_host = rb_strdup(data);
 }
 
+static void
+conf_set_blacklist_type(void *data)
+{
+	conf_parm_t *args = data;
+
+	/* Don't assume we have either if we got here */
+	yy_blacklist_ipv4 = 0;
+	yy_blacklist_ipv6 = 0;
+
+	for (; args; args = args->next)
+	{
+		if (!strcasecmp(args->v.string, "ipv4"))
+			yy_blacklist_ipv4 = 1;
+		else if (!strcasecmp(args->v.string, "ipv6"))
+			yy_blacklist_ipv6 = 1;
+		else
+			conf_report_error("blacklist::type has unknown address family %s",
+					  args->v.string);
+	}
+
+	/* If we have neither, just default to IPv4 */
+	if (!yy_blacklist_ipv4 && !yy_blacklist_ipv6)
+	{
+		conf_report_error("blacklist::type has neither IPv4 nor IPv6 (defaulting to IPv4)");
+		yy_blacklist_ipv4 = 1;
+	}
+}
+
 static void
 conf_set_blacklist_reason(void *data)
 {
@@ -1766,11 +1839,37 @@ conf_set_blacklist_reason(void *data)
 
 	if (yy_blacklist_host && yy_blacklist_reason)
 	{
-		new_blacklist(yy_blacklist_host, yy_blacklist_reason);
+		if (yy_blacklist_ipv6)
+		{
+			/* Make sure things fit (64 = alnum count + dots) */
+			if ((64 + strlen(yy_blacklist_host)) > IRCD_RES_HOSTLEN)
+			{
+				conf_report_error("blacklist::host %s results in IPv6 queries that are too long",
+						  yy_blacklist_host);
+				goto cleanup_bl;
+			}
+		}
+		/* Avoid doing redundant check, IPv6 is bigger than IPv4 --Elizabeth */
+		if (yy_blacklist_ipv4 && !yy_blacklist_ipv6)
+		{
+			/* Make sure things fit (16 = number of nums + dots) */
+			if ((16 + strlen(yy_blacklist_host)) > IRCD_RES_HOSTLEN)
+			{
+				conf_report_error("blacklist::host %s results in IPv4 queries that are too long",
+						  yy_blacklist_host);
+				goto cleanup_bl;
+			}
+		}
+
+		new_blacklist(yy_blacklist_host, yy_blacklist_reason, yy_blacklist_ipv4, yy_blacklist_ipv6);
+
+cleanup_bl:
 		rb_free(yy_blacklist_host);
 		rb_free(yy_blacklist_reason);
 		yy_blacklist_host = NULL;
 		yy_blacklist_reason = NULL;
+		yy_blacklist_ipv4 = 1;
+		yy_blacklist_ipv6 = 0;
 	}
 }
 
@@ -1865,7 +1964,7 @@ conf_call_set(struct TopConf *tc, char *item, conf_parm_t * value, int type)
 
 	/* if it takes one thing, make sure they only passed one thing,
 	   and handle as needed. */
-	if(value->type & CF_FLIST && !cf->cf_type & CF_FLIST)
+	if((value->v.list->type & CF_FLIST) && !(cf->cf_type & CF_FLIST))
 	{
 		conf_report_error
 			("Option %s::%s does not take a list of values.", tc->tc_name, item);
@@ -2011,6 +2110,8 @@ static struct ConfEntry conf_serverinfo_table[] =
 
 	{ "default_max_clients",CF_INT,     NULL, 0, &ServerInfo.default_max_clients },
 
+	{ "nicklen",		CF_INT,     conf_set_serverinfo_nicklen, 0, NULL },
+
 	{ "\0",	0, NULL, 0, NULL }
 };
 
@@ -2024,15 +2125,15 @@ static struct ConfEntry conf_admin_table[] =
 
 static struct ConfEntry conf_log_table[] =
 {
-	{ "fname_userlog", 	CF_QSTRING, NULL, MAXPATHLEN, &ConfigFileEntry.fname_userlog	},
-	{ "fname_fuserlog", 	CF_QSTRING, NULL, MAXPATHLEN, &ConfigFileEntry.fname_fuserlog	},
-	{ "fname_operlog", 	CF_QSTRING, NULL, MAXPATHLEN, &ConfigFileEntry.fname_operlog	},
-	{ "fname_foperlog", 	CF_QSTRING, NULL, MAXPATHLEN, &ConfigFileEntry.fname_foperlog	},
-	{ "fname_serverlog", 	CF_QSTRING, NULL, MAXPATHLEN, &ConfigFileEntry.fname_serverlog	},
-	{ "fname_killlog", 	CF_QSTRING, NULL, MAXPATHLEN, &ConfigFileEntry.fname_killlog	},
-	{ "fname_klinelog", 	CF_QSTRING, NULL, MAXPATHLEN, &ConfigFileEntry.fname_klinelog	},
-	{ "fname_operspylog", 	CF_QSTRING, NULL, MAXPATHLEN, &ConfigFileEntry.fname_operspylog	},
-	{ "fname_ioerrorlog", 	CF_QSTRING, NULL, MAXPATHLEN, &ConfigFileEntry.fname_ioerrorlog },
+	{ "fname_userlog", 	CF_QSTRING, NULL, PATH_MAX, &ConfigFileEntry.fname_userlog	},
+	{ "fname_fuserlog", 	CF_QSTRING, NULL, PATH_MAX, &ConfigFileEntry.fname_fuserlog	},
+	{ "fname_operlog", 	CF_QSTRING, NULL, PATH_MAX, &ConfigFileEntry.fname_operlog	},
+	{ "fname_foperlog", 	CF_QSTRING, NULL, PATH_MAX, &ConfigFileEntry.fname_foperlog	},
+	{ "fname_serverlog", 	CF_QSTRING, NULL, PATH_MAX, &ConfigFileEntry.fname_serverlog	},
+	{ "fname_killlog", 	CF_QSTRING, NULL, PATH_MAX, &ConfigFileEntry.fname_killlog	},
+	{ "fname_klinelog", 	CF_QSTRING, NULL, PATH_MAX, &ConfigFileEntry.fname_klinelog	},
+	{ "fname_operspylog", 	CF_QSTRING, NULL, PATH_MAX, &ConfigFileEntry.fname_operspylog	},
+	{ "fname_ioerrorlog", 	CF_QSTRING, NULL, PATH_MAX, &ConfigFileEntry.fname_ioerrorlog },
 	{ "\0",			0,	    NULL, 0,          NULL }
 };
 
@@ -2090,6 +2191,7 @@ static struct ConfEntry conf_connect_table[] =
 {
 	{ "send_password",	CF_QSTRING,   conf_set_connect_send_password,	0, NULL },
 	{ "accept_password",	CF_QSTRING,   conf_set_connect_accept_password,	0, NULL },
+	{ "fingerprint",	CF_QSTRING,   conf_set_connect_fingerprint,	0, NULL },
 	{ "flags",	CF_STRING | CF_FLIST, conf_set_connect_flags,	0, NULL },
 	{ "host",	CF_QSTRING, conf_set_connect_host,	0, NULL },
 	{ "vhost",	CF_QSTRING, conf_set_connect_vhost,	0, NULL },
@@ -2117,7 +2219,7 @@ static struct ConfEntry conf_general_table[] =
 	{ "default_operstring",	CF_QSTRING, NULL, REALLEN,    &ConfigFileEntry.default_operstring },
 	{ "default_adminstring",CF_QSTRING, NULL, REALLEN,    &ConfigFileEntry.default_adminstring },
 	{ "servicestring",	CF_QSTRING, NULL, REALLEN,    &ConfigFileEntry.servicestring },
-	{ "egdpool_path",	CF_QSTRING, NULL, MAXPATHLEN, &ConfigFileEntry.egdpool_path },
+	{ "egdpool_path",	CF_QSTRING, NULL, PATH_MAX, &ConfigFileEntry.egdpool_path },
 	{ "kline_reason",	CF_QSTRING, NULL, REALLEN, &ConfigFileEntry.kline_reason },
 	{ "identify_service",	CF_QSTRING, NULL, REALLEN, &ConfigFileEntry.identifyservice },
 	{ "identify_command",	CF_QSTRING, NULL, REALLEN, &ConfigFileEntry.identifycommand },
@@ -2132,8 +2234,8 @@ static struct ConfEntry conf_general_table[] =
 	{ "burst_away",		CF_YESNO, NULL, 0, &ConfigFileEntry.burst_away		},
 	{ "caller_id_wait",	CF_TIME,  NULL, 0, &ConfigFileEntry.caller_id_wait	},
 	{ "client_exit",	CF_YESNO, NULL, 0, &ConfigFileEntry.client_exit		},
-	{ "client_flood",	CF_INT,   NULL, 0, &ConfigFileEntry.client_flood	},
 	{ "collision_fnc",	CF_YESNO, NULL, 0, &ConfigFileEntry.collision_fnc	},
+	{ "resv_fnc",		CF_YESNO, NULL, 0, &ConfigFileEntry.resv_fnc		},
 	{ "connect_timeout",	CF_TIME,  NULL, 0, &ConfigFileEntry.connect_timeout	},
 	{ "default_floodcount", CF_INT,   NULL, 0, &ConfigFileEntry.default_floodcount	},
 	{ "default_ident_timeout",	CF_INT, NULL, 0, &ConfigFileEntry.default_ident_timeout		},
@@ -2177,6 +2279,13 @@ static struct ConfEntry conf_general_table[] =
 	{ "use_whois_actually", CF_YESNO, NULL, 0, &ConfigFileEntry.use_whois_actually	},
 	{ "warn_no_nline",	CF_YESNO, NULL, 0, &ConfigFileEntry.warn_no_nline	},
 	{ "use_propagated_bans",CF_YESNO, NULL, 0, &ConfigFileEntry.use_propagated_bans	},
+	{ "client_flood_max_lines",	CF_INT,   NULL, 0, &ConfigFileEntry.client_flood_max_lines	},
+	{ "client_flood_burst_rate",	CF_INT,   NULL, 0, &ConfigFileEntry.client_flood_burst_rate	},
+	{ "client_flood_burst_max",	CF_INT,   NULL, 0, &ConfigFileEntry.client_flood_burst_max	},
+	{ "client_flood_message_num",	CF_INT,   NULL, 0, &ConfigFileEntry.client_flood_message_num	},
+	{ "client_flood_message_time",	CF_INT,   NULL, 0, &ConfigFileEntry.client_flood_message_time	},
+	{ "max_ratelimit_tokens",	CF_INT,   NULL, 0, &ConfigFileEntry.max_ratelimit_tokens	},
+	{ "away_interval",		CF_INT,   NULL, 0, &ConfigFileEntry.away_interval		},
 	{ "\0", 		0, 	  NULL, 0, NULL }
 };
 
@@ -2196,10 +2305,11 @@ static struct ConfEntry conf_channel_table[] =
 	{ "only_ascii_channels", CF_YESNO, NULL, 0, &ConfigChannel.only_ascii_channels },
 	{ "use_except",		CF_YESNO, NULL, 0, &ConfigChannel.use_except		},
 	{ "use_invex",		CF_YESNO, NULL, 0, &ConfigChannel.use_invex		},
-	{ "use_knock",		CF_YESNO, NULL, 0, &ConfigChannel.use_knock		},
 	{ "use_forward",	CF_YESNO, NULL, 0, &ConfigChannel.use_forward		},
+	{ "use_knock",		CF_YESNO, NULL, 0, &ConfigChannel.use_knock		},
 	{ "resv_forcepart",     CF_YESNO, NULL, 0, &ConfigChannel.resv_forcepart	},
 	{ "channel_target_change", CF_YESNO, NULL, 0, &ConfigChannel.channel_target_change	},
+	{ "disable_local_channels", CF_YESNO, NULL, 0, &ConfigChannel.disable_local_channels },
 	{ "\0", 		0, 	  NULL, 0, NULL }
 };
 
@@ -2228,6 +2338,7 @@ newconf_init()
 	add_top_conf("privset", NULL, NULL, conf_privset_table);
 
 	add_top_conf("listen", conf_begin_listen, conf_end_listen, NULL);
+	add_conf_item("listen", "defer_accept", CF_YESNO, conf_set_listen_defer_accept);
 	add_conf_item("listen", "port", CF_INT | CF_FLIST, conf_set_listen_port);
 	add_conf_item("listen", "sslport", CF_INT | CF_FLIST, conf_set_listen_sslport);
 	add_conf_item("listen", "ip", CF_QSTRING, conf_set_listen_address);
@@ -2261,5 +2372,6 @@ newconf_init()
 
 	add_top_conf("blacklist", NULL, NULL, NULL);
 	add_conf_item("blacklist", "host", CF_QSTRING, conf_set_blacklist_host);
+	add_conf_item("blacklist", "type", CF_STRING | CF_FLIST, conf_set_blacklist_type);
 	add_conf_item("blacklist", "reject_reason", CF_QSTRING, conf_set_blacklist_reason);
 }
